@@ -26,16 +26,47 @@
 namespace PowerDevil
 {
 
+using RemainingTimeHistoryList = std::vector<qulonglong>;
+
+/**
+ * Filter data along one dimension using exponential moving average.
+ */
+qulonglong emafilter(const RemainingTimeHistoryList &input)
+{
+    if (input.empty()) {
+        return 0;
+    }
+
+    constexpr double weight = 0.05;
+    qulonglong current = input[0];
+    for (qulonglong n : input) {
+        current = current * (1 - weight) + n * weight;
+    }
+
+    return current;
+}
+
 class BackendInterface::Private
 {
 public:
-    Private() : acAdapterState(UnknownAcAdapterState), batteryState(NoBatteryState), batteryRemainingTime(0),
-                isReady(false), isError(false), isLidClosed(false), isLidPresent(false) {}
+    Private()
+        : acAdapterState(UnknownAcAdapterState)
+        , batteryState(NoBatteryState)
+        , batteryRemainingTime(0)
+        , isReady(false)
+        , isError(false)
+        , isLidClosed(false)
+        , isLidPresent(false)
+    {
+        remainingTimeHistoryRecords.reserve(64);
+    }
     ~Private() {}
 
     AcAdapterState acAdapterState;
     BatteryState batteryState;
     qulonglong batteryRemainingTime;
+    qulonglong smoothedBatteryRemainingTime = 0;
+    RemainingTimeHistoryList remainingTimeHistoryRecords;
     QHash< BrightnessControlType, BrightnessLogic* > brightnessLogic;
     BrightnessControlsList brightnessControlsAvailable;
     Capabilities capabilities;
@@ -73,6 +104,11 @@ qulonglong BackendInterface::batteryRemainingTime() const
     return d->batteryRemainingTime;
 }
 
+qulonglong BackendInterface::smoothedBatteryRemainingTime() const
+{
+    return d->smoothedBatteryRemainingTime;
+}
+
 BackendInterface::BatteryState BackendInterface::batteryState() const
 {
     return d->batteryState;
@@ -140,6 +176,11 @@ void BackendInterface::setAcAdapterState(PowerDevil::BackendInterface::AcAdapter
 {
     d->acAdapterState = state;
     Q_EMIT acAdapterStateChanged(state);
+
+    // AC state has been refreshed, so clear the history records
+    d->remainingTimeHistoryRecords.clear();
+    d->smoothedBatteryRemainingTime = 0;
+    Q_EMIT smoothedBatteryRemainingTimeChanged(0);
 }
 
 void BackendInterface::setBackendHasError(const QString& errorDetails)
@@ -163,6 +204,31 @@ void BackendInterface::setBatteryRemainingTime(qulonglong time)
         d->batteryRemainingTime = time;
         Q_EMIT batteryRemainingTimeChanged(time);
     }
+
+    if (d->acAdapterState == Plugged) {
+        if (d->smoothedBatteryRemainingTime != time) {
+            d->smoothedBatteryRemainingTime = time;
+            Q_EMIT smoothedBatteryRemainingTimeChanged(time);
+        }
+        return;
+    }
+
+    const qulonglong oldValue = d->smoothedBatteryRemainingTime;
+    if (time > 0) {
+        if (d->remainingTimeHistoryRecords.size() == 64) {
+            d->remainingTimeHistoryRecords.erase(d->remainingTimeHistoryRecords.begin());
+        }
+        d->remainingTimeHistoryRecords.emplace_back(time);
+
+        d->smoothedBatteryRemainingTime = emafilter(d->remainingTimeHistoryRecords);
+    } else {
+        // Don't let those samples pollute the history
+        d->smoothedBatteryRemainingTime = time;
+    }
+
+    if (oldValue != d->smoothedBatteryRemainingTime) {
+        Q_EMIT smoothedBatteryRemainingTimeChanged(d->smoothedBatteryRemainingTime);
+    }
 }
 
 void BackendInterface::setBatteryState(PowerDevil::BackendInterface::BatteryState state)
